# -*- coding: utf-8 -*-

from sysactions import get_model, action, get_business_period, get_custom, show_messagebox
from mw_helper import show_message_options_dialog

from actions.util import get_opened_pos_user
from common_close_user import common_close_user
from custom_sysactions import user_authorized_to_execute_action
from typing import Optional
from .. import logger


@action
def sign_out_user(
    pos_id,                     # type: str
    user_id=None,               # type: Optional[str]
    pos_to_close_user=None,     # type: Optional[str]
    current_operator_id=None,   # type: Optional[str]
    auth_data=None,             # type: Optional[str]
):
    # type: (...) -> bool

    if pos_to_close_user is None:
        pos_to_close_user = pos_id

    logger.debug("Logoff POS %d - Iniciando logoff." % int(pos_id))
    model = get_model(posid=pos_to_close_user)

    if user_id is None:
        operator = model.find("Operator")
        if operator is None:
            show_messagebox(pos_id, "$INVALID_USER_OR_PASSWORD", "$INFORMATION")
            return False
        user_id = model.find("Operator").get('id')

    if user_id is None:
        show_messagebox(pos_id, "$INVALID_USER_OR_PASSWORD", "$INFORMATION")
        return False

    pos_user = get_opened_pos_user(
        pos_id=pos_to_close_user,
        user_id=user_id,
    )

    if pos_user is None:
        show_messagebox(pos_id, "$USER_NOT_OPENED", "$INFORMATION")
        return False

    resp = show_message_options_dialog(pos_id, "$OK|$CANCEL", "$INFORMATION", "$CONFIRM_CLOSE_USER|{}"
                                       .format(pos_user.get("name")))
    if resp is None or resp == 1:
        return False

    if not continue_with_opened_tables(
        pos_id=pos_id,
        model=model,
        user_id=user_id,
    ):
        return False

    authorizer = auth_data
    if authorizer is None and current_operator_id and current_operator_id != user_id:
        current_operator = get_opened_pos_user(
            pos_id=pos_to_close_user,
            user_id=current_operator_id,
        )
        authorizer = current_operator.get("name", None) if current_operator is not None else None

    return sign_out_user_close_user(
        pos_id=pos_id,
        model=model,
        user_id=user_id,
        pos_to_close_user=pos_to_close_user,
        authorizer=authorizer,
    )


@user_authorized_to_execute_action
def sign_out_user_close_user(
    pos_id,             # type: str
    model,              # type: str
    user_id,            # type: str
    pos_to_close_user,  # type: str
    auth_data=None,     # type: Optional[str]
    authorizer=None,    # type: Optional[str]
):
    # type: (...) -> bool

    from msgbusboundary import MsgBusTableService as tableService

    authorizer = auth_data if auth_data is not None else authorizer
    return common_close_user(
        pos_id=pos_id,
        model=model,
        user_id=user_id,
        authorizer=authorizer,
        pos_to_close_user=pos_to_close_user,
        table_service=tableService,
    )


def continue_with_opened_tables(pos_id, model, user_id):
    from table_actions import can_close_operator_with_opened_table, has_table_service

    if not has_table_service():
        return True

    is_valid_pod_type = model.find("WorkingMode").get("podType") in ["TS", "FL"]
    is_order_taker = model.find("WorkingMode").get("podFunction") == "OT"
    period = get_business_period(model)

    if is_valid_pod_type and not is_order_taker:
        from tablemgrapi import get_pos_ts
        pos_ts = get_pos_ts(model)
        operator_table = pos_ts.listTables(user_id)
        for table in operator_table:
            table_in_current_business_period = table["businessPeriod"].replace("-", "") == period
            table_is_opened = table['statusDescr'] in ("InProgress", "Seated", "Waiting2BSeated", "Totaled")
            if table_is_opened and table_in_current_business_period:
                if can_close_operator_with_opened_table:
                    ret = show_messagebox(pos_id, "$HAS_OPENED_TABLES_CONTINUE", buttons="$YES|$NO")
                    return ret == 0
                else:
                    show_messagebox(pos_id, "$HAS_OPENED_TABLES")
                    return False

    return True



import printerconstants
import printer_text_formatter
from xml.etree import cElementTree as eTree
from typing import Optional


import sysactions

REPRINT_CONSTANT = "####REPRINT_TEXT####"


def parse_constants(
    printer_text,       # type: str
    pos_id=None,        # type: Optional[eTree.XML]
    is_reprint=False,   # type: Optional[bool]
):
    # type: (...) -> str

    if not is_reprint:
        return printer_text.replace(REPRINT_CONSTANT, "")

    model = sysactions.get_model(posid=pos_id)
    reprint_text = _get_reprint_text(model=model)
    return printer_text.replace(REPRINT_CONSTANT, reprint_text)


def _get_reprint_text(
    model,  # type: eTree.XML
):
    # type: (...) -> str

    reprint_text = rollback_printer_default_font()

    if model is not None:
        reprint_translation = sysactions.translate_message(
            model=model,
            message_key="REPRINT_TEXT_MESSAGE",
        )
    else:
        reprint_translation = "****REIMPRESSAO****"

    reprint_text += printerconstants.TXT_BOLD_ON
    reprint_text += printerconstants.TXT_2HEIGHT
    reprint_text += printerconstants.TXT_2WIDTH
    reprint_text += printer_text_formatter.center_text(
        text=reprint_translation,
        line_size=printerconstants.COLS/2,
    )

    reprint_text += rollback_printer_default_font()

    return reprint_text


def rollback_printer_default_font():
    # type: () -> str

    return "{}{}{}{}{}".format(
        printerconstants.TXT_NORMAL,
        printerconstants.TXT_BOLD_OFF,
        printerconstants.TXT_FONT_A,
        printerconstants.TXT_ALIGN_LT,
        printerconstants.TXT_INVERT_OFF
    )



# -*- coding: utf-8 -*-

from pos_model import TenderType

from models import NfeTender


class PosTenderToNfeTender(object):
    @staticmethod
    def convert(pos_tender):
        # type: (int) -> int

        tender_map = {
            TenderType.cash: NfeTender.dinheiro,
            TenderType.credit: NfeTender.cartao_credito,
            TenderType.debit: NfeTender.cartao_debito
        }

        nfe_tender = tender_map.get(pos_tender, NfeTender.outros)
        return nfe_tender



import os

import pyscripts
from application.configurator import Configurator
from application.model import Configurations
from application.repository import NDiscountRepository
from helper import import_pydevd
from mbcontextmessagehandler import MbContextMessageBus, MbContextMessageHandler
from messagehandlerbuilder import ComponentMessageHandlerBuilder
from msgbus import MBEasyContext
import sysactions


def main():
    # type: () -> None

    loader_path = os.environ["LOADERCFG"]

    import_pydevd(loader_path, 9167)

    configurations = _get_configurations(loader_path)

    mb_context = MBEasyContext(configurations.service_name)
    sysactions.mbcontext = mb_context
    pyscripts.mbcontext = mb_context
    message_bus = MbContextMessageBus(mb_context)

    import wrappers
    wrappers.set_wrapper()

    repository = NDiscountRepository(configurations, message_bus)
    message_handler_builder = ComponentMessageHandlerBuilder(configurations, repository, message_bus)

    message_handler = MbContextMessageHandler(
        message_bus,
        configurations.service_name,
        configurations.service_name,
        configurations.required_services,
        message_handler_builder
    )

    repository.build_singletons()
    message_handler.handle_events()


def _get_configurations(loader_path):
    # type: (str) -> Configurations

    configurator = Configurator(loader_path)

    configurator.read_loader_configurations()
    configurator.configure_logger()

    return configurator.configs


if __name__ == "__main__":
    main()



import datetime  # noqa


class Transfer(object):
    def __init__(self, transfer_type, transfer_operator, transfer_date, transfer_value, transfer_cash_in_cash_out):
        # type: (unicode, int, datetime, float) -> None
        self.transfer_type = transfer_type
        self.transfer_operator = transfer_operator
        self.transfer_date = transfer_date
        self.transfer_value = transfer_value
        self.transfer_cash_in_cash_out = transfer_cash_in_cash_out



import os

from msgbus import MBEasyContext

from application.configurator import Configurator
from application.model import ListenedEvents, BenefitAppliers
from application.model.configuration import Configurations
from application.model.repository.benefit_applier import DefaultBenefitApplierRepository
from application.repository.pos import DBRepository
from application.repository.pos.benefitapplier import (
    LoyaltyCouponApplierRepository,
    NDiscountCouponApplierRepository,
)
from application.repository.pos.utils import fill_benefit_appliers_repositories
from helper import import_pydevd
from mbcontextmessagehandler import (
    MbContextMessageBus,
    MbContextMessageHandler,
)
from messagehandlerbuilder import ComponentMessageHandlerBuilder
from typing import Dict


def main():
    # type: () -> None

    loader_path = os.environ["LOADERCFG"]

    import_pydevd(loader_path, 9166)

    configurations = _get_configurations(loader_path)

    mb_context = MBEasyContext(configurations.service_name)
    message_bus = MbContextMessageBus(mb_context)

    db_repository = DBRepository(configurations, message_bus)
    benefit_appliers_repositories = _get_benefit_appliers_repositories(configurations, message_bus)

    component_message_handler_builder = ComponentMessageHandlerBuilder(configurations,
                                                                       db_repository,
                                                                       benefit_appliers_repositories)

    mb_context_message_handler = MbContextMessageHandler(message_bus,
                                                         configurations.service_name,
                                                         configurations.service_name,
                                                         configurations.required_services,
                                                         component_message_handler_builder)

    mb_context_message_handler.subscribe_reentrant_events([ListenedEvents.ORDER_MODIFIED.value])

    mb_context_message_handler.handle_events()


def _get_benefit_appliers_repositories(configs, message_bus):
    # type: (Configurations, MbContextMessageBus) -> Dict[BenefitAppliers: DefaultBenefitApplierRepository]

    benefit_appliers_repositories = {}
    default_params = [configs, message_bus]

    for benefit_applier in configs.supported_benefit_appliers:
        applier_name = benefit_applier.split("|")[0]
        if applier_name == BenefitAppliers.N_DISCOUNT:
            benefit_appliers_repositories[applier_name] = NDiscountCouponApplierRepository(*default_params)
        elif applier_name == BenefitAppliers.LOYALTY:
            benefit_appliers_repositories[applier_name] = LoyaltyCouponApplierRepository(*default_params)

    fill_benefit_appliers_repositories(benefit_appliers_repositories)

    return benefit_appliers_repositories


def _get_configurations(loader_path):
    # type: (str) -> Configurations

    configurator = Configurator(loader_path)

    configurator.read_loader_configurations()
    configurator.configure_logger()

    return configurator.configs


if __name__ == "__main__":
    main()



def build_response(products):
    response_list = {"rupturedItems": []}
    for product_code in products:
        response_list["rupturedItems"].append(int(product_code))
    return response_list


class GetRupturedProductsResponse(object):
    def __init__(self, ruptured_products):
        # type: (list) -> None

        self.ruptured_products = build_response(ruptured_products)



try:
    import unittest2 as unittest
except ImportError:
    import unittest
try:
    from unittest import mock
except ImportError:
    import mock

from qrcode import run_example


class ExampleTest(unittest.TestCase):

    @mock.patch('PIL.Image.Image.show')
    def runTest(self, mock_show):
        run_example()
        mock_show.assert_called_with()



import sys

if sys.version_info[0] < 3:
    import inspect2 as inspect
else:
    import inspect

import json
from datetime import datetime, date
from typing import Any, Union, List
from uuid import UUID

import iso8601
from httpmessageprocessor import ApiRequest
from httpmessageprocessorutil.automapped.exception import PropertyError

from ._util import to_camel_case, convert_underscore_to_hyphen
from .exception import BodyNotJson, PropertyNotSent


class ApiRequestParser(object):
    def __init__(self, request_type, extra_mapping):
        self.request_type = request_type
        self.extra_mapping = extra_mapping if extra_mapping is not None else {}
        self.context_mappings = []

        if self.request_type is not None:
            self.attributes = self.build_type_attributes(self.request_type)
        self.json_body = None

    def parse(self, api_request):
        # type: (ApiRequest) -> Any
        if self.request_type is None:
            return None
        constructor_args = self.get_variables_from_request(api_request)
        if self.extra_mapping.get("removeNotSent", False):
            full_constructor_args = {}
            for attr_name in self.attributes.keys():
                if attr_name in constructor_args:
                    full_constructor_args[attr_name] = constructor_args[attr_name]
                else:
                    full_constructor_args[attr_name] = None
        else:
            full_constructor_args = constructor_args
        value = self.request_type(**full_constructor_args)
        self.remove_not_sent(full_constructor_args, constructor_args, value)

        return value

    def remove_not_sent(self, full_constructor_args, constructor_args, value):
        if self.extra_mapping.get("removeNotSent", False):
            for attr_name in full_constructor_args:
                if getattr(value, attr_name) is None and attr_name not in constructor_args:
                    delattr(value, attr_name)

    def get_variables_from_request(self, api_request):
        constructor_args = {}
        not_found_variables = []
        self.get_variables_from_path_and_query(api_request, constructor_args, not_found_variables)
        if len(not_found_variables) > 0:
            self.get_variables_from_body(api_request, constructor_args, not_found_variables)
        return constructor_args

    def get_variables_from_path_and_query(self, api_request, constructor_args, not_found_variables):
        for attr_name in self.attributes.keys():
            value = self.get_from_path_or_query(api_request, attr_name)
            if value is not None:
                constructor_args[attr_name] = value
            else:
                not_found_variables.append(attr_name)

    def get_variables_from_body(self, api_request, constructor_args, not_found_variables):
        body_variables = self.get_from_body(api_request, not_found_variables)
        for variable_name in body_variables:
            if variable_name not in constructor_args:
                constructor_args[variable_name] = body_variables[variable_name]

    def build_type_attributes(self, cls):
        attributes = {}
        if hasattr(cls, "_real_cls"):
            init_params = inspect.signature(cls._real_cls.__init__).parameters
        else:
            init_params = inspect.signature(cls.__init__).parameters
        for arg_name in init_params.keys():
            value = init_params[arg_name]
            if arg_name == "self":
                continue
            attrs = {}
            if value.annotation != inspect.Signature.empty:
                attrs["type"] = value.annotation
            else:
                attrs["type"] = Any

            if hasattr(cls, "_cls_attrs") and arg_name in cls._cls_attrs:
                for key in cls._cls_attrs[arg_name].keys():
                    attrs[key] = cls._cls_attrs[arg_name][key]

            extra_mapping = self.get_current_mapping()
            if extra_mapping is not None and arg_name in extra_mapping:
                for key in extra_mapping[arg_name].keys():
                    extra_mapping_value = extra_mapping[arg_name][key]
                    if not isinstance(extra_mapping_value, dict):
                        attrs[key] = extra_mapping_value

            attributes[arg_name] = attrs

        return attributes

    def get_from_path_or_query(self, api_request, attr_name):
        attr_map_properties = self.attributes[attr_name]
        name = attr_name
        if "name" in attr_map_properties:
            name = attr_map_properties["name"]

        if "location" in attr_map_properties:
            location = attr_map_properties["location"]
            if location == "query":
                selected_dict = api_request.query_parameters
                selected_dict_name = "query string"
            elif location == "path":
                selected_dict = api_request.path_parameters
                selected_dict_name = "path parameters"
            elif location == "header":
                selected_dict = api_request.headers
                selected_dict_name = "header parameters"
            elif location == "body":
                return None
            else:
                raise Exception("Invalid Location: {}".format(location))

            found_name = name
            if name not in selected_dict:
                camel_name = to_camel_case(name)
                if camel_name not in selected_dict:
                    raise Exception("Attribute {} not found in {}".format(name, selected_dict_name))
                else:
                    found_name = camel_name

            if "type" in attr_map_properties:
                return self.transform_value(selected_dict[found_name], attr_map_properties["type"])
            else:
                return selected_dict[name]
        else:
            return self.get_attr_value_from_path_or_query(attr_name, name, api_request)

    def get_attr_value_from_path_or_query(self, attr_name, variable_name, api_request):
        value = self.get_attr_value_from_dicts(variable_name, api_request)
        if value is None:
            return value
        if "type" in self.attributes[attr_name]:
            return self.transform_value(value, self.attributes[attr_name]["type"])
        return value

    def get_attr_value_from_dicts(self, attr_name, api_request):
        if attr_name in api_request.path_parameters:
            return api_request.path_parameters[attr_name]
        camel_name = to_camel_case(attr_name)
        if camel_name in api_request.path_parameters:
            return api_request.path_parameters[camel_name]

        if attr_name in api_request.query_parameters:
            return api_request.query_parameters[attr_name]
        if camel_name in api_request.query_parameters:
            return api_request.query_parameters[camel_name]

    def get_from_body(self, api_request, variable_names):
        body_str, force_json = self.encode_body(api_request)

        body_variables = {}
        for attr_name in variable_names:
            attr_map_properties = self.attributes[attr_name]
            name = attr_name
            if "name" in attr_map_properties:
                name = attr_map_properties["name"]

            body_type = Any
            if "type" in attr_map_properties:
                body_type = attr_map_properties["type"]

            try:
                body_variables[attr_name] = self.build_object_from_body(body_type, body_str, name, force_json)
            except PropertyNotSent:
                pass
            except BodyNotJson:
                raise
            except Exception as ex:
                raise PropertyError(attr_name)
        return body_variables

    @staticmethod
    def encode_body(api_request):
        encoding_name = "utf-8"
        force_json = False
        if "Content-Type" in api_request.headers:
            if api_request.headers["Content-Type"].startswith("application/json"):
                force_json = True

            separator_index = api_request.headers["Content-Type"].find(";")
            if separator_index >= 0:
                charset_specification = api_request.headers["Content-Type"][separator_index + 1:].strip()
                parts = charset_specification.split("=")
                if len(parts) > 1:
                    encoding_name = parts[1]
        body_str = api_request.body.decode(encoding_name)
        return body_str, force_json

    def build_object_from_body(self, type, body, attr_name, force_json):
        try:
            if self.json_body is None:
                self.json_body = json.loads(body)
            if attr_name in self.json_body:
                try:
                    if attr_name in self.extra_mapping:
                        self.context_mappings.append(self.extra_mapping[attr_name])
                    else:
                        self.context_mappings.append(None)
                    return self.transform_value(self.json_body[attr_name], type)
                finally:
                    self.context_mappings.pop()

            camel_attr_name = to_camel_case(attr_name)
            if camel_attr_name in self.json_body:
                return self.transform_value(self.json_body[camel_attr_name], type)
            hyphen_attr_name = convert_underscore_to_hyphen(attr_name)
            if hyphen_attr_name in self.json_body:
                return self.transform_value(self.json_body[hyphen_attr_name], type)

            return self.transform_value(None, type, True)
        except ValueError:
            if force_json:
                raise BodyNotJson(body)

            if body == "":
                return None

            body_type = str
            if attr_name is not None and self.is_scalar_type(type):
                body_type = type
            return self.transform_value(body, body_type)

    @staticmethod
    def is_scalar_type(type):
        return type == int or type == float or type == bool or type == str or type == datetime or type == date

    def transform_value(self, value, new_type, property_not_sent=False):
        if new_type == Any:
            return value

        if new_type == str:
            if value is None:
                raise Exception("str type does not accept None values")
            return str(value)

        if new_type == int:
            return int(value)

        if new_type == float:
            return float(value)

        if new_type == bool:
            if isinstance(value, str):
                return value.lower() == "true"
            if isinstance(value, bool):
                return value
            if isinstance(value, int):
                return value != 0
            raise Exception("Value cannot be converted to bool: {}".format(value))

        if new_type == datetime or new_type == date:
            return iso8601.parse_date(value)

        if new_type == UUID:
            return UUID(hex=value)

        if hasattr(new_type, "__origin__"):
            if new_type.__origin__ == list or new_type.__origin__ == List:
                inner_type = new_type.__args__[0]
                if not isinstance(value, list):
                    raise Exception("Value is not a list: {}".format(value))

                ret = []
                for item in value:
                    ret.append(self.transform_value(item, inner_type))
                return ret

            if new_type.__origin__ == dict:
                key_type = new_type.__args__[0]
                value_type = new_type.__args__[1]
                if not isinstance(value, dict):
                    raise Exception("Value is not a dict: {}".format(value))
                ret = {}
                for json_key in value.keys():
                    ret[self.transform_value(json_key, key_type)] = self.transform_value(value[json_key], value_type)
                return ret

            if new_type.__origin__ == Union:
                if len(new_type.__args__) == 2 and \
                        (isinstance(None, new_type.__args__[1]) or isinstance(None, new_type.__args__[0])):
                    if value is None:
                        if property_not_sent and self.extra_mapping.get("removeNotSent", False):
                            raise PropertyNotSent()
                        return None
                    else:
                        if isinstance(None, new_type.__args__[1]):
                            return self.transform_value(value, new_type.__args__[0])
                        else:
                            return self.transform_value(value, new_type.__args__[1])

            raise Exception("Unsupported type: {}".format(new_type))

        type_attributes = self.build_type_attributes(new_type)
        constructor_args = {}
        full_constructor_args = {}
        for attr_name in type_attributes.keys():
            search_name = self.get_search_name(attr_name)
            try:
                if search_name in value:
                    constructor_args[attr_name] = self.transform_value(value[search_name], type_attributes[attr_name]["type"])
                else:
                    camel_attr_name = to_camel_case(search_name)
                    hyphen_attr_name = convert_underscore_to_hyphen(search_name)
                    if camel_attr_name in value:
                        constructor_args[attr_name] = self.transform_value(value[camel_attr_name],
                                                                           type_attributes[attr_name]["type"])
                    elif hyphen_attr_name in value:
                        constructor_args[attr_name] = self.transform_value(value[hyphen_attr_name],
                                                                           type_attributes[attr_name]["type"])
                    else:
                        try:
                            constructor_args[attr_name] = self.transform_value(None,
                                                                               type_attributes[attr_name]["type"],
                                                                               property_not_sent=True)
                        except PropertyNotSent:
                            full_constructor_args[attr_name] = None
            except Exception as ex:
                raise PropertyError(attr_name)

        for attr_name in constructor_args.keys():
            full_constructor_args[attr_name] = constructor_args[attr_name]

        value = new_type(**full_constructor_args)
        self.remove_not_sent(full_constructor_args, constructor_args, value)
        return value

    def get_search_name(self, attr_name):
        current_mapping = self.get_current_mapping()
        if current_mapping is not None and attr_name in current_mapping and "name" in current_mapping[attr_name]:
            return current_mapping[attr_name]["name"]
        return attr_name

    def get_current_mapping(self):
        if len(self.context_mappings) > 0:
            return self.context_mappings[-1]
        else:
            return self.extra_mapping



from application.model import (
    BenefitAppliers,
    OperationDescription,
)
from application.model.configuration import Configurations
from application.model.repository.benefit_applier import DefaultBenefitApplierRepository
from application.repository.pos import DBRepository
from typing import (
    Optional,
    Dict,
)


class BurnBenefitInteractor(object):

    def __init__(self, configs, db_repository, benefit_appliers_repositories):
        # type: (Configurations, DBRepository, Dict[BenefitAppliers: DefaultBenefitApplierRepository]) -> None

        self.configs = configs
        self.logger = self.configs.logger
        self.db_repository = db_repository
        self.benefit_appliers_repositories = benefit_appliers_repositories

    def burn_benefit(self, benefit_id, originator_pos_id, benefit_applier, retry_quantity=0):
        # type: (str, int, BenefitAppliers, Optional[int]) -> None

        success = False
        applier_name = benefit_applier.split("|")[0]
        try:
            message = "Burning benefit. BenefitId: {}; OriginatorPosId: {}; BenefitApplier: {}; Try: {}"
            self.logger.info(message.format(benefit_id, originator_pos_id, applier_name, retry_quantity))
            if benefit_applier not in [None, 'None', '']:
                self.benefit_appliers_repositories[applier_name].burn_benefit(
                    benefit_id=benefit_id,
                    pos_id=originator_pos_id,
                )
            success = True

        except Exception as _:
            self.logger.exception("Error burning benefit: {}".format(benefit_id))
        finally:
            self.db_repository.manage_operation_status(
                success,
                benefit_id,
                originator_pos_id,
                OperationDescription.BURNED,
                applier_name,
                retry_quantity
            )



import datetime

from commons.dto import ReportColumnDefinition, AlignTypes
from commons.report import Formatter
from commons.util import TableReport
from reports_app.opendaysummary.dto import OpenDaySummaryDto, OpenDaySummaryHeaderDto  # noqa


class OpenDayFormatter(Formatter):
    def __init__(self):
        self.report_lines = []
        self.translated_messages = []

    def format_report(self, open_day_summary_dto, translator):
        # type: (OpenDaySummaryDto) -> None
        self.translated_messages = translator.translate_labels(
            ["$OPEN_DAY",
             "$STORE",
             "$POS",
             "$OPERATOR",
             "$CURRENT_DATE",
             "$BUSINESS_DATE",
             "$AUTHORIZER"
             ],
            open_day_summary_dto.open_day_summary_header_dto.pos_id)

        report = self._append_header(open_day_summary_dto.open_day_summary_header_dto)
        report += self._append_body(open_day_summary_dto.open_day_summary_body_dto)

        report_bytes = report.decode("utf-8").encode("utf-8")
        return report_bytes

    def _append_header(self, open_day_summary_header_dto):
        # type: (OpenDaySummaryHeaderDto) -> str
        self._append_line_separator()
        self._append_title()
        self._append_store_pos(open_day_summary_header_dto.store_id, open_day_summary_header_dto.pos_id)
        self._append_operator(open_day_summary_header_dto.operator_id)
        self._append_current_date()
        self._append_business_date(open_day_summary_header_dto.business_date)

        table_formatter = TableReport([
            ReportColumnDefinition(width=17, fill_with_char='.'),
            ReportColumnDefinition(width=21, before_text=': ')
        ])

        return table_formatter.format(self.report_lines)

    def _append_body(self, open_day_summary_body_dto):
        self.report_lines = []

        self._append_empty_line()

        self._append_authorizer(open_day_summary_body_dto.authorizer)

        self._append_line_separator()

        table_formatter = TableReport([
            ReportColumnDefinition(width=17, fill_with_char='.'),
            ReportColumnDefinition(width=21, before_text=': ')
        ])

        return table_formatter.format(self.report_lines)

    def _append_line_separator(self):
        self.report_lines.append([ReportColumnDefinition(
            fill_with_char='='
        )])

    def _append_title(self):
        title = self.translated_messages["$OPEN_DAY"]

        self.report_lines.append([ReportColumnDefinition(
            text=title,
            align=AlignTypes.CENTER
        )])

    def _append_store_pos(self, store_id, pos_id):
        self.report_lines.append([self.translated_messages["$STORE"] + "/" + self.translated_messages["$POS"], (str(store_id) + '/' + str(pos_id))])

    def _append_operator(self, operator_id):
        self.report_lines.append([self.translated_messages["$OPERATOR"], str(operator_id)])

    def _append_current_date(self):
        self.report_lines.append([self.translated_messages["$CURRENT_DATE"], datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S")])

    def _append_business_date(self, business_date):
        self.report_lines.append([self.translated_messages["$BUSINESS_DATE"], business_date])

    def _append_empty_line(self):
        self.report_lines.append([ReportColumnDefinition(fill_with_char=' ')])

    def _append_authorizer(self, authorizer):
        self.report_lines.append([self.translated_messages["$AUTHORIZER"], str(authorizer)])



import json
from typing import List

from production.box import ViewBox


class GetPrintersInfoInteractor(object):
    def __init__(self, view_boxes):
        # type: (List[ViewBox]) -> None
        self.view_boxes = view_boxes

    def execute(self):
        # type: () -> str
        info = []
        for view_box in self.view_boxes:
            name = getattr(view_box.view, 'printer', None)
            if name and name not in info:
                info.append(name)

        return json.dumps(info)



import logging
from datetime import datetime

from bustoken import (
    TK_ELECTRONIC_PAYMENT_CONNECTIVITY_TEST,
    TK_ELECTRONIC_PAYMENT_GET_ELECTRONIC_TYPES,
    TK_ELECTRONIC_PAYMENT_GET_NAME,
)
from msgbus import (
    TK_SYS_ACK,
    TK_SYS_NAK
)
from sysactions import (
    action,
    show_messagebox
)
from typing import (
    Any,
    Union
)

from .. import mb_context
from actions.eft import get_electronic_payment_component_name

logger = logging.getLogger("PosActions")


@action
def check_tef_connectivity(pos_id, show_dialog="false"):
    try:
        show_dialog = show_dialog.lower() == "true"

        component_name = get_electronic_payment_component_name(pos_id=pos_id)

        data = "{};{};{}".format(pos_id, datetime.now().strftime("%Y%m%d"), datetime.now().strftime('%H:%M:%S'))

        ret = mb_context.MB_EasySendMessage(component_name, TK_ELECTRONIC_PAYMENT_CONNECTIVITY_TEST, data=data)

        if show_dialog:
            if ret.token == TK_SYS_NAK:
                show_messagebox(pos_id, "$SITEF_IS_NOT_AVAILABLE", "$ERROR")
            else:
                show_messagebox(pos_id, "$SITEF_OK")

        return str(ret.data).lower()
    except Exception as ex:
        logger.warning("Error verifying if Electronic Payments is available. Exception: {}".format(ex.message))

        if show_dialog:
            show_messagebox(pos_id, "$SITEF_SERVER_ERROR|{}".format(str(pos_id)), "$ERROR")

        return "false"


@action
def get_available_tefs(
    pos_id      # type: Any[Union[str, int]]
):
    # type: (...) -> str

    component_name = get_electronic_payment_component_name(pos_id=pos_id)
    if not component_name:
        return "{}"

    try:
        ret = mb_context.MB_EasySendMessage(
            dest_name=component_name,
            token=TK_ELECTRONIC_PAYMENT_GET_ELECTRONIC_TYPES,
            data=""
        )
        if ret.token == TK_SYS_ACK:
            return ret.data
        return "{}"
    except Exception as _:
        return "{}"


@action
def get_eft_processor_name(
    pos_id,     # type: str
):
    # type: (...) -> str

    try:
        component_name = get_electronic_payment_component_name(pos_id=pos_id)
        response = mb_context.MB_EasySendMessage(
            dest_name=component_name,
            token=TK_ELECTRONIC_PAYMENT_GET_NAME,
            data=pos_id,
        )
        if response.token == TK_SYS_ACK:
            return str(response.data)

    except Exception as ex:
        logger.warning("Error get eft processor name. Exception: {}".format(ex))

    return ""



from logging import Logger

from production.repository import ProductRepository
from typing import Union, List

from ._ComboFilterBox import ComboFilterBox
from ._ProductionBox import ProductionBox


class ShowOnJitComboFilterBox(ComboFilterBox):
    def __init__(self, name, sons, product_repository, logger=None):
        # type: (str, Union[ProductionBox, List[ProductionBox]], ProductRepository, Logger) -> None
        items_to_keep = product_repository.get_combos_to_keep()
        super(ShowOnJitComboFilterBox, self).__init__(name, sons, items_to_keep, logger)



from commons.report import Generator
from deliveryreport import DeliveryReport
from deliveryreport.repository import (
    DeliveryOrderRepository,
    DeliveryProductRepository,
)
from dto import DeliveryHeaderDto, DeliveryBodyDto, DeliveryGeneratorDto


class DeliveryGenerator(Generator):
    def __init__(
        self,
        order_repository,       # type: DeliveryOrderRepository
        product_repository,     # type: DeliveryProductRepository
        order_id,               # type: str
        show_customer_phone,    # type: bool
    ):
        # type: (...) -> None

        self.order_repository = order_repository
        self.product_repository = product_repository
        self.order_id = order_id
        self.show_customer_phone = show_customer_phone

    def generate_data(self):
        delivery_report = DeliveryReport(self.order_repository, self.product_repository)
        order = delivery_report.get_delivery_order(self.order_id)
        products = delivery_report.get_products()
        cfh_products = delivery_report.get_cfh_products()
        header = DeliveryHeaderDto(order)
        body = DeliveryBodyDto(order)
        return DeliveryGeneratorDto(
            header=header,
            body=body,
            products=products,
            cfh_products=cfh_products,
            show_customer_phone=self.show_customer_phone,
        )



# -*- coding: utf-8 -*-

import xml.sax.saxutils as saxutils
from cStringIO import StringIO

from production.model import ProductionOrder, Item, StateEvent

_PICKLE_PROTOCOL = 2  # Pickle protocol version to use ("2" is the best one at the time this is written)


def _validxml(name, clazz):
    """private - checks if an attribute should be written in the XML"""
    if not name.startswith("_"):
        value = getattr(clazz, name)
        if (not hasattr(value, '__iter__')) and (not callable(value)):
            return True
    return False


def utf(value):
    """retrieves the utf8 representation of a value"""
    return value.encode("UTF-8") if isinstance(value, unicode) else str(value)


class OrderXml(object):

    """ class OrderXml(object)
    Class used to generate an XML representation of a ProductionOrder
    """
    _OUT_TYPES = (str, unicode, int, long, float, bool,)
    _ORDER_ATTRS = [name for name in dir(ProductionOrder) if _validxml(name, ProductionOrder)]
    # level attribute generated when writing the xml to avoid gaps in the sequence
    _ITEM_ATTRS = [name for name in dir(Item) if _validxml(name, Item) and name != 'level']
    _EVENT_ATTRS = [name for name in dir(StateEvent) if _validxml(name, StateEvent)]

    def to_xml(self, order):
        """ obj.to_xml(order) -> str
        Creates and XML representation of the given order.
        @param order: {ProductionOrder} - the production box to generate XML from
        @return {str} - The XML output
        """
        io = StringIO()
        io.write('<?xml version="1.0" encoding="UTF-8"?>\n<ProductionOrder')
        self._write_attributes(io, self._ORDER_ATTRS, order)
        io.write('>\n')
        # Write custom storage
        self._write_custom_storage(io, order, 1)
        # Write items recursively
        io.write('  <Items>\n')
        for item in order.items:
            self._write_item(io, item, 0, 2)
        io.write('  </Items>\n')
        # Write the order properties
        io.write('  <Properties>\n')
        for key in order.properties:
            self._write_property(io, key, order.properties[key], 2)
        io.write('  </Properties>\n')
        # Write the state history
        io.write('  <StateHistory>\n')
        for event in order.state_history:
            io.write('    <StateEvent')
            self._write_attributes(io, self._EVENT_ATTRS, event)
            io.write('/>\n')
        io.write('  </StateHistory>\n')
        io.write('</ProductionOrder>')
        return io.getvalue()

    def _write_comments(self, io, item, level):
        level = int(level)
        level += 1
        for comment_id in item.comments:
            ident = (level * '  ')
            comment = item.comments[comment_id]
            io.write('{0}<Comment id="{1}" comment={2}/>\n'.format(ident, comment_id, utf(saxutils.quoteattr(comment.text))))

    def _write_item(self, io, item, item_level, indent_level):
        """private - writes an Item recursively"""
        indent_space = (indent_level * '  ')
        io.write('%s<Item' % indent_space)
        io.write(' %s=%s' % ('level', saxutils.quoteattr(str(item_level))))
        self._write_attributes(io, self._ITEM_ATTRS, item)
        io.write('>\n')
        indent_level += 1
        # write custom item properties
        if len(item.properties) > 0:
            prop_space = (indent_level * '  ')
            io.write('%s<Properties>\n' % prop_space)
            prop_level = indent_level + 1
            for key in item.properties:
                self._write_property(io, key, item.properties[key], prop_level)
            io.write('%s</Properties>\n' % prop_space)
        # Write custom storage
        self._write_custom_storage(io, item, indent_level)
        # Write comments
        self._write_comments(io, item, indent_level)
        # Write sub-items recursively
        item_level += 1
        for subitem in item.items:
            self._write_item(io, subitem, item_level, indent_level)
        io.write('%s</Item>\n' % indent_space)

    def _write_property(self, io, key, value, level):
        """private - writes an Item recursively"""
        ident = (int(level) * '  ')
        io.write('{0}<Property key="{1}" value={2}/>\n'.format(ident, key, utf(saxutils.quoteattr(value))))

    def _write_attributes(self, io, names, obj):
        """private - writes a list of attributes"""
        out_types = self._OUT_TYPES
        quoteattr = saxutils.quoteattr
        for name in names:
            value = getattr(obj, name)
            if type(value) in out_types:
                io.write(' %s=%s' % (name, quoteattr(utf(value))))

    def _write_custom_storage(self, io, obj, level):
        """private - writes any custom storage data of an object"""
        storage = obj._storage
        ident = (int(level) * '  ')
        quoteattr = saxutils.quoteattr
        for context, dic in storage.iteritems():
            context = quoteattr(utf(context))
            for key, value in dic.iteritems():
                if not key.startswith("_"):
                    key = quoteattr(utf(key))
                    if type(value) in (str, unicode):
                        value = utf(value)
                    else:
                        value = repr(value).decode('string_escape')
                    value = quoteattr(utf(value))
                    io.write('%s<Custom context=%s key=%s value=%s/>\n' % (ident, context, key, value))


class ProdPersistException(Exception):
    pass


class OrderNotFoundException(ProdPersistException):
    pass



from typing import Dict

from helper import json_serialize

from externals.util import BaseThread, LoggerUtil

logger = LoggerUtil.get_logger_name(__name__)


class LogManager(BaseThread):
    def __init__(self, name, counter, context="", sleep_time=60):
        # type: (str, Dict[str, int], str, int) -> None
        
        super(LogManager, self).__init__()

        self.name = name
        self.counter = counter

        self.context = context
        self.sleep_time = sleep_time

        logger.info("Creating LogManager [{}] with [{}]s of sleep time".format(self.context, sleep_time))

    def run(self):
        # type: () -> None
        
        logger.info("Starting LogManager [{}]".format(self.context))
        try:
            while self.running:
                self._print_counter(self.counter, self.name)
                self.sleep(self.sleep_time)
        except Exception as _:
            logger.exception("LogManager [{}] EXCEPTION:".format(self.context))

    def stop(self):
        # type: () -> None
    
        logger.warning("Stopping LogManager [{}]".format(self.context))

        self.running = False
        with self.sleep_condition:
            self.sleep_condition.notify()

    def _print_counter(self, d, name):
        # type: (Dict[str, int], str) -> None
        
        counted = json_serialize(d)
        total = sum(d.values())
        d.clear()
        logger.info("[{}][{}]: [{}] -> [{}]".format(self.context, name, total, counted))



from logging import Logger

from production.box import CourseBox
from production.repository import ProductRepository
from typing import List, Optional, Union, Tuple, Any

from ._ProductionBox import ProductionBox


class CustomParamCourseBox(CourseBox):
    def __init__(self, name, sons, product_repository, publish_scheduler, logger=None):
        # type: (str, Optional[Union[List[ProductionBox]], ProductionBox], ProductRepository, List[Tuple[str, int]], Logger) -> None # noqa

        courses = product_repository.get_all_courses()
        course_products = product_repository.get_courses_products()
        configs = []
        if len(courses) > 0 and len(course_products) > 0:
            for course in courses:
                course_name = course.name
                description = course.description
                products = course_products[course_name] if course_name in course_products else {}
                configuration = CourseBox.CourseConfiguration(
                    name=course_name,
                    description=description,
                    products=products,
                    next_course_wait_time=course.delay,
                )
                configs.append(configuration)

        super(CustomParamCourseBox, self).__init__(name, sons, configs, publish_scheduler, logger)



from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder
from production.box import TagFilterBox


class TagFilterBoxBuilder(BoxBuilder):
    def __init__(self, loggers):
        super(TagFilterBoxBuilder, self).__init__(loggers)

    def build(
        self,
        config,     # type: BoxConfiguration
    ):
        # type: (BoxConfiguration) -> TagFilterBox
        inherit_comments = self.get_extra(config, "InheritComments", "false").lower() == "true"

        return TagFilterBox(
            config.name,
            config.sons,
            self.get_extra(config, "AllowedTags", None),
            self.get_extra(config, "ForbiddenTags", None),
            self.get_extra(config, "IgnoreTags", None),
            self.get_extra(config, "AllowComboTags", 'False'),
            self.get_extra(config, "KeepItemWithSons", 'False'),
            inherit_comments,
            self.get_logger(config),
        )



# encoding: utf-8
from datetime import datetime


class JustificationDiscountHeaderDto(object):

    def __init__(
        self,
        operator,       # type: str
        pos_id,         # type: str
        start_date,     # type: str
        end_date,       # type: str
    ):
        # type: (...) -> None

        self.title = "$JUSTIFICATION_DISCOUNT_REPORT"
        self.pos_id = pos_id
        self.current_date = datetime.now().strftime("%d/%m/%Y %H:%M:%S")
        self.operator = operator if operator is not None else "$ALL_OPERATORS"
        self.start_date = datetime.strptime(start_date, '%Y%m%d').strftime('%d/%m/%y')
        self.end_date = datetime.strptime(end_date, '%Y%m%d').strftime('%d/%m/%y')
